@@ -37,8 +37,10 @@
 #include <KLocale>
 
 #include <QDebug>
+#include <QIcon>
+#include <QLocale>
 #include <QProcess>
-#include <qtest.h>
+#include <QTest>
 
 QTEST_MAIN(IncidenceFormatterTest)
 
@@ -50,6 +52,8 @@ void IncidenceFormatterTest::initTestCase()
     QStandardPaths::setTestModeEnabled(true);
     GrantleeTemplateManager::instance()->setTemplatePath(QStringLiteral(TEST_TEMPLATE_PATH));
     GrantleeTemplateManager::instance()->setPluginPath(QStringLiteral(TEST_PLUGIN_PATH));
+    QIcon::setThemeName(QStringLiteral("oxygen"));
+    QLocale::setDefault(QLocale(QStringLiteral("C")));
 }
 
 void IncidenceFormatterTest::testRecurrenceString()
@@ -133,12 +137,12 @@ void IncidenceFormatterTest::testRecurrenceString()
              i18n("Recurs every 2 hours until %1", endDateStr));
 
     r3->addExDateTime(kdt.addSecs(1 * 60 * 60));
-    QString hourStr = QLocale::system().toString(QTime(13, 0), QLocale::ShortFormat);
+    QString hourStr = QLocale().toString(QTime(13, 0), QLocale::ShortFormat);
     QCOMPARE(IncidenceFormatter::recurrenceString(e3),
              i18n("Recurs every 2 hours until %1 (excluding %2)", endDateStr, hourStr));
 
     r3->addExDateTime(kdt.addSecs(3 * 60 * 60));
-    QString hourStr2 = QLocale::system().toString(QTime(15, 0), QLocale::ShortFormat);
+    QString hourStr2 = QLocale().toString(QTime(15, 0), QLocale::ShortFormat);
     QCOMPARE(IncidenceFormatter::recurrenceString(e3),
              i18n("Recurs every 2 hours until %1 (excluding %2,%3)", endDateStr, hourStr, hourStr2));
 
@@ -209,6 +213,8 @@ bool IncidenceFormatterTest::compareHtml(const QString &name)
         content.replace(QRegExp(QStringLiteral("\"file:[^\"]*[/(?:%2F)]([^\"/(?:%2F)]*)\"")), QStringLiteral("\"file:\\1\""));
         // emoticons give us absolute paths without file:
         content.replace(QRegExp(QStringLiteral("src=\"/[^\"]*[/(?:%2F)]([^\"/(?:%2F)]*)\"")), QStringLiteral("src=\"\\1\""));
+        // icon filename extensions depend on used theme, Oxygen has PNG, Breeze has SVG
+        content.replace(QRegExp(QStringLiteral(".(png|svg)\"")), QStringLiteral("\""));
         if (!f.open(QIODevice::WriteOnly | QIODevice::Truncate)) {
             return false;
         }
@@ -248,7 +254,7 @@ void IncidenceFormatterTest::testErrorTemplate()
     const QString expected = QStringLiteral(
                                  "<h1>Template parsing error</h1>\n"
                                  "<b>Template:</b> broken-template.html<br>\n"
-                                 "<b>Error message:</b> Unclosed tag in template broken-template.html. Expected one of: (else endif), line 2, broken-template.html");
+                                 "<b>Error message:</b> Unclosed tag in template broken-template.html. Expected one of: (elif else endif), line 2, broken-template.html");
 
     QCOMPARE(html, expected);
 }
@@ -363,3 +369,60 @@ void IncidenceFormatterTest::testDisplayViewFreeBusy()
 
     cleanup(name);
 }
+
+void IncidenceFormatterTest::testFormatIcalInvitation_data()
+{
+    QTest::addColumn<QString>("name");
+
+    QTest::newRow("itip-journal") << QStringLiteral("itip-journal");
+    QTest::newRow("itip-journal-delegation-request") << QStringLiteral("itip-journal-delegation-request");
+    QTest::newRow("itip-journal-delegation-reply") << QStringLiteral("itip-journal-delegation-reply");
+    QTest::newRow("itip-journal-declined-reply") << QStringLiteral("itip-journal-declined-reply");
+    QTest::newRow("itip-journal-tentative-reply") << QStringLiteral("itip-journal-tentative-reply");
+    QTest::newRow("itip-journal-accepted-reply") << QStringLiteral("itip-journal-accepted-reply");
+
+    QTest::newRow("itip-todo") << QStringLiteral("itip-todo");
+    QTest::newRow("itip-todo-with-start") << QStringLiteral("itip-todo-with-start");
+    QTest::newRow("itip-todo-delegation-request") << QStringLiteral("itip-todo-delegation-request");
+    QTest::newRow("itip-todo-delegation-reply") << QStringLiteral("itip-todo-delegation-reply");
+    QTest::newRow("itip-todo-declined-reply") << QStringLiteral("itip-todo-declined-reply");
+    QTest::newRow("itip-todo-tentative-reply") << QStringLiteral("itip-todo-tentative-reply");
+    QTest::newRow("itip-todo-accepted-reply") << QStringLiteral("itip-todo-accepted-reply");
+
+    QTest::newRow("itip-event-with-html-description") << QStringLiteral("itip-event-with-html-description");
+    QTest::newRow("itip-event-with-recurrence-attachment-reminder") << QStringLiteral("itip-event-with-recurrence-attachment-reminder");
+    QTest::newRow("itip-event-multiday-allday") << QStringLiteral("itip-event-multiday-allday");
+    QTest::newRow("itip-event-multiday") << QStringLiteral("itip-event-multiday");
+    QTest::newRow("itip-event-allday") << QStringLiteral("itip-event-allday");
+    QTest::newRow("itip-event") << QStringLiteral("itip-event");
+    QTest::newRow("itip-event-request") << QStringLiteral("itip-event-request");
+    QTest::newRow("itip-event-counterproposal") << QStringLiteral("itip-event-counterproposal");
+    QTest::newRow("itip-event-counterproposal-declined") << QStringLiteral("itip-event-counterproposal-declined");
+    QTest::newRow("itip-event-delegation-request") << QStringLiteral("itip-event-delegation-request");
+    QTest::newRow("itip-event-delegation-reply") << QStringLiteral("itip-event-delegation-reply");
+    QTest::newRow("itip-event-declined-reply") << QStringLiteral("itip-event-delegation-reply");
+    QTest::newRow("itip-event-tentative-reply") << QStringLiteral("itip-event-tentative-reply");
+    QTest::newRow("itip-event-accepted-reply") << QStringLiteral("itip-event-accepted-reply");
+}
+
+void IncidenceFormatterTest::testFormatIcalInvitation()
+{
+    QFETCH(QString, name);
+
+    KCalCore::MemoryCalendar::Ptr calendar(new KCalCore::MemoryCalendar(KDateTime::Spec(KDateTime::UTC)));
+    InvitationFormatterHelper helper;
+
+    QFile eventFile(QStringLiteral(TEST_DATA_DIR "/%1.ical").arg(name));
+    QVERIFY(eventFile.exists());
+    QVERIFY(eventFile.open(QIODevice::ReadOnly));
+    const QByteArray data = eventFile.readAll();
+
+    const QString html = IncidenceFormatter::formatICalInvitation(QString::fromUtf8(data),
+                                                                  calendar,
+                                                                  &helper);
+
+    QVERIFY(validateHtml(name, html));
+    QVERIFY(compareHtml(name));
+
+    cleanup(name);
+}
